import 'dart:io';

import 'package:example/models/post_model.dart';
import 'package:example/network/api_client.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter_simple_template_bloc/flutter_simple_template_bloc.dart';

class DataRepository implements ApiClient {
  final Dio _dio = Dio();
  final String _baseUrl = "https://jsonplaceholder.typicode.com";

  late ApiService _apiService;

  DataRepository() {
    _dio.interceptors.addAll([
      if (kDebugMode) PrettyDioLogger(responseBody: true),
    ]);
    _apiService = ApiService(_dio, _baseUrl);
  }

  @override
  ApiService get apiService => _apiService;

  @override
  Future<DataState<List<PostModel>>> getPost() async {
    try {
      final response = await _apiService.get('/posts');
      final List<dynamic> responseData = response.data;
      final List<PostModel> posts =
          responseData.map((json) => PostModel.fromJson(json)).toList();
      return DataSuccess(posts);
    } on DioException catch (e) {
      return DataFailed(e);
    }
  }

  @override
  Future<DataState<PostModel>> createPost(dynamic data) async {
    try {
      final response = await _apiService.postRequest('/posts', data);
      return _handleContent<PostModel>(
        response,
        statusCodeApprove: HttpStatus.created,
      );
    } on DioException catch (e) {
      return DataFailed(e);
    }
  }

  @override
  Future<DataState<PostModel>> updatePost(dynamic data) async {
    try {
      final response = await _apiService.putRequest('/posts/1', data);
      return _handleContent<PostModel>(response);
    } on DioException catch (e) {
      return DataFailed(e);
    }
  }

  @override
  Future<DataState<PostModel>> deletePost() {
    throw UnimplementedError();
  }

  DataState<T> _handleContent<T>(Response response, {int? statusCodeApprove}) {
    statusCodeApprove ??= HttpStatus.ok;
    if (response.data != null && response.statusCode == statusCodeApprove) {
      return DataSuccess<T>(response.data);
    } else {
      return DataFailed(
        DioException(
          requestOptions: response.requestOptions,
          error: response.statusMessage,
          response: response,
          type: DioExceptionType.badResponse,
        ),
      );
    }
  }
}
